#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2020-2021, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# CBOR Content for CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8769.txt
#

from pyasn1.type import univ


id_ct_cbor = univ.ObjectIdentifier('1.2.840.113549.1.9.16.1.44')


id_ct_cborSequence = univ.ObjectIdentifier('1.2.840.113549.1.9.16.1.45')
